var bizagiUtil = require('bz-util');
var REQUIRED = bizagiUtil.REQUIRED;
var ERROR = bizagiUtil.error;
var RESPONSE = bizagiUtil.getResponse;
var xlsx = REQUIRED('xlsx');
 
/**
 * @author Bizagi
 */ 
 
 function convertCell(cell){
	
	var result ;
	
	if (cell != undefined)
		result = {
		value:cell.v,
		formattedText:cell.w,
		type:cell.t,
		formula:cell.f
	};
	
	return result;
}

function invoke(globals, actionName, data, authenticationType, LOG, callback) {
LOG.debug(['[bz-excel.', actionName, '] globals: ', globals, ' --- data: ', data]);
    
	var error = bizagiUtil.validator.requiredParameters({
	    'cellNameFrom':data.inputs.input.cellNameFrom,
	    'cellNameTo':data.inputs.input.cellNameTo,
	    'workbookdata':data.inputs.input.workbookdata,'worksheetname':data.inputs.input.worksheetname, 'callback':callback});
    if (error){
        LOG.error(['[net-excel-connector.', actionName, '] Error: ', error]);
        callback(error);
        return;
    } 	
    
    LOG.info(['[bz-excel.', actionName, '] invoke... getCellData: { cellname: ', data.inputs.input.cellname, ', worksheetname:', data.inputs.input.worksheetname, ', workbookdata:... }']);
    
    var reply;
    var array = [];
    try{        
        var bufferedData= new Buffer(data.inputs.input.workbookdata, 'base64');
        var workbook = xlsx.read(bufferedData.toString("binary"), {type:"binary"});
        var worksheet = workbook.Sheets[data.inputs.input.worksheetname];
        var range = {
          s: xlsx.utils.decode_cell(data.inputs.input.cellNameFrom),
          e: xlsx.utils.decode_cell(data.inputs.input.cellNameTo)
        }
        
        for (var R = range.s.r; R <= range.e.r; ++R) {
          for (var C = range.s.c; C <= range.e.c; ++C) {
            var cell_address = {
              c: C,
              r: R
            };
            /* if an A1-style address is needed, encode the address */
            var cell_ref = xlsx.utils.encode_cell(cell_address);
            var valueCell = convertCell(worksheet[cell_ref]);
            console.log(cell_ref, ':',valueCell);
            var value =  valueCell ? valueCell.value : valueCell;
            array.push({cellname: cell_ref, value: value})
          }
        }
    }
    catch (e){
        var error = { statusCode:500, message: e.toString()};
    }
    
    if(error){
        error = RESPONSE(null, error, error.statusCode, ERROR('EXCEL.ERROR',[error.statusCode, error.message]));
        LOG.error(['[bz-excel.', actionName, '] Error: ', error]);
		callback(error);
	}	
	else{
        reply =RESPONSE(array, null, 200);
        LOG.debug(['[bz-excel.', actionName, '] Response Success with response... ', reply]);
		callback(reply);
	}	 
}

exports.invoke = invoke;